APP_TITLE = "paita"
APP_SUBTITLE = "Python AI Textual Assistant"
APP_AUTHOR = "paita"
AI_PERSONA_PREFIX = "AI Persona:"
AI_STREAMING = "Streaming"
AI_MODEL_KWARGS = "Extra arguments to pass to model"
AI_MAX_TOKENS = "Max tokens"
AI_HISTORY_DEPTH = "History depth"
AI_N = "Number of response messages"
AI_RAG_ENABLED = "RAG Enabled"
AI_RAG_SOURCE = "Source"
AI_RAG_SOURCE_MAX_DEPTH = "Max depth"

APP_LIST_AI_SERVICES_MODELS = "Checking available AI Services and AI Models"

APP_RAG_PROCESS_DOCUMENTS = "Processing documents..."
APP_ERROR_NO_AI_SERVICES_OR_MODELS = "No available AI Services or AI Models founds"

APP_DIALOG_BUTTON_EXIT = "Exit"
